@@ -10,11 +10,14 @@ for sub-dependencies
     a. "first found, wins" (where the order is breadth first)
 """
 
+from __future__ import annotations
+
 import logging
 import sys
 from collections import defaultdict
+from collections.abc import Iterable
 from itertools import chain
-from typing import DefaultDict, Iterable, List, Optional, Set, Tuple
+from typing import Optional
 
 from pipenv.patched.pip._vendor.packaging import specifiers
 from pipenv.patched.pip._vendor.packaging.requirements import Requirement
@@ -49,12 +52,12 @@ from pipenv.patched.pip._internal.utils.packaging import check_requires_python
 
 logger = logging.getLogger(__name__)
 
-DiscoveredDependencies = DefaultDict[Optional[str], List[InstallRequirement]]
+DiscoveredDependencies = defaultdict[Optional[str], list[InstallRequirement]]
 
 
 def _check_dist_requires_python(
     dist: BaseDistribution,
-    version_info: Tuple[int, int, int],
+    version_info: tuple[int, int, int],
     ignore_requires_python: bool = False,
 ) -> None:
     """
@@ -117,7 +120,7 @@ class Resolver(BaseResolver):
         self,
         preparer: RequirementPreparer,
         finder: PackageFinder,
-        wheel_cache: Optional[WheelCache],
+        wheel_cache: WheelCache | None,
         make_install_req: InstallRequirementProvider,
         use_user_site: bool,
         ignore_dependencies: bool,
@@ -125,7 +128,7 @@ class Resolver(BaseResolver):
         ignore_requires_python: bool,
         force_reinstall: bool,
         upgrade_strategy: str,
-        py_version_info: Optional[Tuple[int, ...]] = None,
+        py_version_info: tuple[int, ...] | None = None,
     ) -> None:
         super().__init__()
         assert upgrade_strategy in self._allowed_strategies
@@ -152,7 +155,7 @@ class Resolver(BaseResolver):
         self._discovered_dependencies: DiscoveredDependencies = defaultdict(list)
 
     def resolve(
-        self, root_reqs: List[InstallRequirement], check_supported_wheels: bool
+        self, root_reqs: list[InstallRequirement], check_supported_wheels: bool
     ) -> RequirementSet:
         """Resolve what operations need to be done
 
@@ -174,7 +177,7 @@ class Resolver(BaseResolver):
         # exceptions cannot be checked ahead of time, because
         # _populate_link() needs to be called before we can make decisions
         # based on link type.
-        discovered_reqs: List[InstallRequirement] = []
+        discovered_reqs: list[InstallRequirement] = []
         hash_errors = HashErrors()
         for req in chain(requirement_set.all_requirements, discovered_reqs):
             try:
@@ -192,9 +195,9 @@ class Resolver(BaseResolver):
         self,
         requirement_set: RequirementSet,
         install_req: InstallRequirement,
-        parent_req_name: Optional[str] = None,
-        extras_requested: Optional[Iterable[str]] = None,
-    ) -> Tuple[List[InstallRequirement], Optional[InstallRequirement]]:
+        parent_req_name: str | None = None,
+        extras_requested: Iterable[str] | None = None,
+    ) -> tuple[list[InstallRequirement], InstallRequirement | None]:
         """Add install_req as a requirement to install.
 
         :param parent_req_name: The name of the requirement that needed this
@@ -242,8 +245,8 @@ class Resolver(BaseResolver):
             return [install_req], None
 
         try:
-            existing_req: Optional[InstallRequirement] = (
-                requirement_set.get_requirement(install_req.name)
+            existing_req: InstallRequirement | None = requirement_set.get_requirement(
+                install_req.name
             )
         except KeyError:
             existing_req = None
@@ -323,9 +326,7 @@ class Resolver(BaseResolver):
             req.should_reinstall = True
         req.satisfied_by = None
 
-    def _check_skip_installed(
-        self, req_to_install: InstallRequirement
-    ) -> Optional[str]:
+    def _check_skip_installed(self, req_to_install: InstallRequirement) -> str | None:
         """Check if req_to_install should be skipped.
 
         This will check if the req is installed, and whether we should upgrade
@@ -377,7 +378,7 @@ class Resolver(BaseResolver):
         self._set_req_to_reinstall(req_to_install)
         return None
 
-    def _find_requirement_link(self, req: InstallRequirement) -> Optional[Link]:
+    def _find_requirement_link(self, req: InstallRequirement) -> Link | None:
         upgrade = self._is_upgrade_allowed(req)
         best_candidate = self.finder.find_requirement(req, upgrade)
         if not best_candidate:
@@ -488,7 +489,7 @@ class Resolver(BaseResolver):
         self,
         requirement_set: RequirementSet,
         req_to_install: InstallRequirement,
-    ) -> List[InstallRequirement]:
+    ) -> list[InstallRequirement]:
         """Prepare a single requirements file.
 
         :return: A list of additional InstallRequirements to also install.
@@ -511,7 +512,7 @@ class Resolver(BaseResolver):
             ignore_requires_python=self.ignore_requires_python,
         )
 
-        more_reqs: List[InstallRequirement] = []
+        more_reqs: list[InstallRequirement] = []
 
         def add_req(subreq: Requirement, extras_requested: Iterable[str]) -> None:
             # This idiosyncratically converts the Requirement to str and let
@@ -569,7 +570,7 @@ class Resolver(BaseResolver):
 
     def get_installation_order(
         self, req_set: RequirementSet
-    ) -> List[InstallRequirement]:
+    ) -> list[InstallRequirement]:
         """Create the installation order.
 
         The installation order is topological - requirements are installed
@@ -580,7 +581,7 @@ class Resolver(BaseResolver):
         # installs the user specified things in the order given, except when
         # dependencies must come earlier to achieve topological order.
         order = []
-        ordered_reqs: Set[InstallRequirement] = set()
+        ordered_reqs: set[InstallRequirement] = set()
 
         def schedule(req: InstallRequirement) -> None:
             if req.satisfied_by or req in ordered_reqs:
